#!/usr/bin/env python

"""Perform decorrelation on MWUS simulation output."""

import argparse

from origamipy import conditions
from origamipy import decorrelate
from origamipy import files
from origamipy import outputs
from origamipy import utility


def main():
    args = parse_args()
    system_file = files.JSONStructInpFile(args.system_filename)
    staple_lengths = utility.calc_staple_lengths(system_file)
    fileformatter = construct_fileformatter()
    filebase = f"{args.outs_dir}/{args.filebase}"
    reps_all_conditions = conditions.construct_mwus_conditions(
        args.windows_filename,
        args.bias_functions_filename,
        args.reps,
        args.start_run,
        args.temp,
        args.itr,
        args.staple_m,
        fileformatter,
        filebase,
        staple_lengths,
        False,
    )
    sim_collections = []
    for rep in range(args.reps):
        rep_sim_collections = outputs.create_sim_collections(
            filebase,
            reps_all_conditions[rep],
            rep,
            args.start_run,
            args.end_run,
            use_mod_ops=True,
        )
        sim_collections.append(rep_sim_collections)

    decor_outs = decorrelate.DecorrelatedOutputs(
        sim_collections, rep_conditions_equal=False
    )
    decor_outs.perform_decorrelation(
        args.skip, detect_equil=args.detect_equil, g=args.sample
    )
    decor_outs.apply_masks(filebase)
    decor_outs.write_decors_to_files(filebase)


def construct_fileformatter():
    specs = [conditions.ConditionsFileformatSpec("bias", "{}")]
    return conditions.ConditionsFileformatter(specs)


def parse_args():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter
    )
    parser.add_argument("system_filename", type=str, help="System file")
    parser.add_argument("filebase", type=str, help="Base name for files")
    parser.add_argument("outs_dir", type=str, help="outs directory")
    parser.add_argument("windows_filename", type=str, help="Windows filename")
    parser.add_argument(
        "bias_functions_filename", type=str, help="Bias functions filename"
    )
    parser.add_argument("temp", type=float, help="Temperature (K)")
    parser.add_argument("staple_m", type=float, help="Staple molarity (mol/V)")
    parser.add_argument("stack_ene", type=float, help="Stacking energy (kb K)")
    parser.add_argument("skip", type=int, help="Number of steps to skip")
    parser.add_argument("reps", type=int, help="Number of reps")
    parser.add_argument("start_run", type=int, help="Run to concatenate from")
    parser.add_argument("end_run", type=int, help="Run to concatenate to")
    parser.add_argument("itr", type=int, help="US iteration")
    parser.add_argument(
        "--detect_equil",
        type=bool,
        default=False,
        help="Detect and truncate equilibration period",
    )
    parser.add_argument(
        "--sample", type=int, default=None, help="Statistical inefficiency"
    )

    return parser.parse_args()


if __name__ == "__main__":
    main()
